import os
import sys
import shutil
import tempfile
from itertools import chain
from dektools.file import sure_dir, write_file, read_text, remove_path, sure_parent_dir, normal_path, \
    format_path_desc, read_file, split_ext, path_ext
from dektools.zip import compress_files, decompress_files
from dektools.output import pprint
from dektools.net import get_available_port
from ..redirect import search_bin_by_path_tree
from ...utils.beep import sound_notify


def _is_true(x):
    if isinstance(x, str):
        x = x.lower()
    return x not in {'false', '0', 'none', 'null', '', ' ', False, 0, None, b'', b'\0'}


def _parent_dir(path, num=1):
    cursor = path
    for i in range(int(num)):
        cursor = os.path.dirname(cursor)
    return cursor


def _list_dir_one(path):
    item = next(iter(os.listdir(path)), '')
    return normal_path(os.path.join(path, item)) if item else item


def _which(x):
    return shutil.which(x) or ''


def _where(x):
    for path in chain(iter([os.getcwd()]), iter(os.environ['PATH'].split(os.pathsep))):
        fp = os.path.join(path, x)
        if os.path.exists(fp):
            return fp
    return ''


default_methods = {
    'echo': print,
    'echox': lambda *x: print(*x, file=sys.stderr),
    'pp': pprint,
    'path': {
        'exists': os.path.exists,
        'parent': _parent_dir,
        'abs': normal_path,
        'fullname': os.path.basename,
        'name': lambda x: split_ext(x)[0],
        'ext': path_ext,
        'desc': format_path_desc,
        'md': sure_dir,
        'mdp': sure_parent_dir,
        'mdt': lambda x=None: tempfile.mkdtemp(prefix=x),
        'lsa': lambda x='.': [normal_path(os.path.join(x, y)) for y in os.listdir(x)],
        'lso': lambda x='.': _list_dir_one(x),
        'ls': lambda x='.': os.listdir(x),
        'rm': remove_path,
        'wf': write_file,
        'rt': read_text,
        'rf': read_file,
    },
    'cd': os.chdir,
    'cwd': lambda: os.getcwd(),
    'which': _which,
    'where': _where,
    'pybin': lambda x, p=None: search_bin_by_path_tree(p or os.getcwd(), x, False),

    'compress': compress_files,
    'decompress': decompress_files,

    'me': lambda x: x,
    'true': lambda x=True: _is_true(x),
    'false': lambda x=False: not _is_true(x),

    'equal': lambda x, y: x == y,
    'notequal': lambda x, y: x != y,
    'beep': lambda x=True: sound_notify(x),

    'net': {
        'port': get_available_port
    },
}
